import 'dart:math';

import 'package:flutter/material.dart';

import 'package:flutter_gen/gen_l10n/l10n.dart';
import 'package:future_loading_dialog/future_loading_dialog.dart';
import 'package:vrouter/vrouter.dart';

import 'package:fluffychat/config/themes.dart';
import 'package:fluffychat/msc/extension_spaces_advanced/src/space_hierarchy_model.dart';
import 'package:fluffychat/utils/fluffy_share.dart';
import 'package:fluffychat/widgets/avatar.dart';
import 'package:fluffychat/widgets/content_banner.dart';
import 'package:fluffychat/widgets/matrix.dart';

class RecommendedRoomListItem extends StatelessWidget {
  final SpaceRoomsChunk room;

  const RecommendedRoomListItem({Key? key, required this.room})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.transparent,
      child: ListTile(
        leading: Avatar(
          mxContent: room.avatarUrl,
          name: room.name,
        ),
        title: Row(
          children: <Widget>[
            Expanded(
              child: Text(
                room.name ?? '',
                maxLines: 1,
                overflow: TextOverflow.ellipsis,
                softWrap: false,
                style: TextStyle(
                  fontWeight: FontWeight.bold,
                  color: Theme.of(context).textTheme.bodyText1!.color,
                ),
              ),
            ),
            // number of joined users
            Padding(
              padding: const EdgeInsets.only(left: 4.0),
              child: Text.rich(
                TextSpan(children: [
                  WidgetSpan(
                      child: Tooltip(
                        child: const Icon(Icons.people),
                        message: L10n.of(context)!
                            .numberRoomMembers(room.numJoinedMembers),
                      ),
                      alignment: PlaceholderAlignment.middle,
                      baseline: TextBaseline.alphabetic),
                  TextSpan(text: ' ${room.numJoinedMembers}')
                ]),
                style: TextStyle(
                  fontSize: 13,
                  color: Theme.of(context).textTheme.bodyText2!.color,
                ),
              ),
            ),
          ],
        ),
        subtitle: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Expanded(
              child: Text(
                room.topic ?? 'topic',
                softWrap: false,
                maxLines: 1,
                overflow: TextOverflow.ellipsis,
                style: TextStyle(
                  color: Theme.of(context).textTheme.bodyText2!.color,
                ),
              ),
            ),
          ],
        ),
        onTap: () => showModalBottomSheet(
            context: context,
            builder: (c) => RecommendedRoomPreview(room: room)),
      ),
    );
  }
}

class RecommendedRoomPreview extends StatelessWidget {
  final SpaceRoomsChunk room;

  const RecommendedRoomPreview({Key? key, required this.room})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final client = Matrix.of(context).client;
    return Center(
      child: SizedBox(
        width: min(
            MediaQuery.of(context).size.width, FluffyThemes.columnWidth * 1.5),
        child: Material(
          elevation: 4,
          child: SafeArea(
            child: Scaffold(
              extendBodyBehindAppBar: true,
              appBar: AppBar(
                elevation: 0,
                backgroundColor:
                    Theme.of(context).scaffoldBackgroundColor.withOpacity(0.5),
                leading: IconButton(
                  icon: const Icon(Icons.arrow_downward_outlined),
                  onPressed: Navigator.of(context, rootNavigator: false).pop,
                  tooltip: L10n.of(context)!.close,
                ),
                title: Text(room.name ?? L10n.of(context)!.chatDetails),
              ),
              body: Column(
                children: [
                  Expanded(
                    child: ContentBanner(
                      mxContent: room.avatarUrl,
                      defaultIcon: Icons.group,
                      client: client,
                    ),
                  ),
                  if (room.topic != null)
                    ListTile(
                      title: Text(L10n.of(context)!.groupDescription),
                      subtitle: Text(room.topic!),
                    ),
                  ListTile(
                    title: Text(L10n.of(context)!.link),
                    subtitle: Text(room.roomId),
                    trailing: Icon(Icons.adaptive.share_outlined),
                    onTap: () => FluffyShare.share(room.roomId, context),
                  ),
                  ListTile(
                    title: ElevatedButton(
                        onPressed: () async {
                          final client = Matrix.of(context).client;
                          final joinedFuture = client.onSync.stream
                              .where((u) =>
                                  u.rooms?.join?.containsKey(room.roomId) ??
                                  false)
                              .first;
                          final router = VRouter.of(context);
                          await client.joinRoomById(room.roomId);

                          await showDialog(
                              context: context,
                              builder: (c) =>
                                  LoadingDialog(future: () => joinedFuture));
                          Navigator.of(context).pop();
                          router.toSegments(['rooms', room.roomId]);
                        },
                        child: Text(L10n.of(context)!.joinRoom)),
                  )
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}
